package com.abi.qms.platform.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.abi.qms.platform.dao.entity.QrPackageSapOrderItem;
import com.abi.qms.platform.dao.mapper.QrPackageSapOrderItemMapper;
import com.abi.qms.platform.dto.req.BatchImportPackageSapOrderItemReq;
import com.abi.qms.platform.service.QrPackageSapOrderItemService;
import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.api.exception.ErrorCodeEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;

/**
 * @author: fangxinjian
 * @date: 2021/06/07 09:30
 * @description:
 */
@Service
@Slf4j
public class QrPackageSapOrderItemServiceImpl implements QrPackageSapOrderItemService {

    @Autowired
    private QrPackageSapOrderItemMapper qrPackageSapOrderItemMapper;

    @Override
    public void batchImportPackageSapOrderItem(BatchImportPackageSapOrderItemReq req) {

        List<BatchImportPackageSapOrderItemReq.BatchImportSapOrderItem> sapOrderItemList = req.getBatchImportSapOrderItemList();
        for (BatchImportPackageSapOrderItemReq.BatchImportSapOrderItem item : sapOrderItemList) {

            try {
                QrPackageSapOrderItem sapOrderItem = new QrPackageSapOrderItem();
                sapOrderItem.setSapOrderNo(item.getSapOrderNo());
                sapOrderItem.setSapMaterialId(item.getSapMaterialId());
                sapOrderItem.setSapCoverFactoryId(item.getSapCoverFactoryId());
                sapOrderItem.setSapBeerFactoryId(item.getSapBeerFactoryId());
                sapOrderItem.setSapMaterialName(item.getSapMaterialName());
                sapOrderItem.setPurchaseCount(item.getPurchaseCount());

                saveQrPackageSapOrderItem(sapOrderItem);
            } catch (Exception e) {

                log.error("导入sap订单明细数据失败",e);
            }
        }

    }

    @Override
    public void batchUpdatePackageSapOrderItem(BatchImportPackageSapOrderItemReq req) {

        List<BatchImportPackageSapOrderItemReq.BatchImportSapOrderItem> sapOrderItemList = req.getBatchImportSapOrderItemList();
        for (BatchImportPackageSapOrderItemReq.BatchImportSapOrderItem item : sapOrderItemList) {

            try {

                updateQrPackageSapOrderItem(item);
            } catch (Exception e) {

                log.error("修改Sap订单明细数据失败",e);
            }
        }

    }

    @Override
    public void batchDeletePackageSapOrderItem(BatchImportPackageSapOrderItemReq req) {
        List<BatchImportPackageSapOrderItemReq.BatchImportSapOrderItem> sapOrderItemList = req.getBatchImportSapOrderItemList();
        for (BatchImportPackageSapOrderItemReq.BatchImportSapOrderItem itemReq : sapOrderItemList) {

            //根据订单号和物料ID查询订单明细信息
            QrPackageSapOrderItem sapOrderItem = getQrPackageSapOrderItemInfo(itemReq.getSapOrderNo(), itemReq.getSapMaterialId());
            if (ObjectUtil.isNull(sapOrderItem)) {
                log.info("Sap物料订单明细订单号:{}sap物料id:{}不存在",itemReq.getSapOrderNo(),itemReq.getSapMaterialId());
                return;
            }

            //获取需要删除的采购数量
            long newPurchaseCount = itemReq.getPurchaseCount() == null ? 0 : itemReq.getPurchaseCount();
            //获取当前的采购数量
            long nowPurchaseCount = sapOrderItem.getPurchaseCount() == null ? 0 : sapOrderItem.getPurchaseCount();
            long purchaseCount = nowPurchaseCount - newPurchaseCount;

            //采购数量等于0的情况下 就删除此记录
            if (purchaseCount == 0) {
                qrPackageSapOrderItemMapper.deleteById(sapOrderItem.getId());
            }

            sapOrderItem.setPurchaseCount(purchaseCount);
            qrPackageSapOrderItemMapper.updateById(sapOrderItem);
        }

    }

    private void updateQrPackageSapOrderItem(BatchImportPackageSapOrderItemReq.BatchImportSapOrderItem item) {

        //根据sap订单号和物料ID查询物料订单明细信息
        QrPackageSapOrderItem qrPackageSapOrderItem = getQrPackageSapOrderItemInfo(item.getSapOrderNo(),item.getSapMaterialId());

        QrPackageSapOrderItem sapOrderItem = new QrPackageSapOrderItem();
        sapOrderItem.setSapOrderNo(item.getSapOrderNo());
        sapOrderItem.setSapMaterialId(item.getSapMaterialId());
        sapOrderItem.setSapCoverFactoryId(item.getSapCoverFactoryId());
        sapOrderItem.setSapBeerFactoryId(item.getSapBeerFactoryId());
        sapOrderItem.setSapMaterialName(item.getSapMaterialName());

        //判断订单信息是否存在如果不存在就新增一条数据
        if (ObjectUtil.isNull(qrPackageSapOrderItem)) {
            sapOrderItem.setPurchaseCount(item.getPurchaseCount());
            saveQrPackageSapOrderItem(sapOrderItem);
        }

        sapOrderItem.setId(qrPackageSapOrderItem.getId());

        //判断物料ID是否修改 如果没有修改 采购数量=（新的采购数量-老的采购数量）+当前的采购数量
        if (qrPackageSapOrderItem.getSapMaterialId().equals(item.getOriSapMaterialId())) {
            sapOrderItem.setPurchaseCount((item.getPurchaseCount() - item.getOriPurchaseCount()) + qrPackageSapOrderItem.getPurchaseCount());
        } else {
            //如果物料ID修改 采购数量等于当前采购数量减去老的采购数量再加上新的采购数量
            long purchaseCount = (qrPackageSapOrderItem.getPurchaseCount() - item.getOriPurchaseCount()) + item.getPurchaseCount();
            //如果采购数量为0就删除此订单号数据
            if (purchaseCount == 0) {
                qrPackageSapOrderItemMapper.deleteById(qrPackageSapOrderItem.getId());
                return;
            }
            sapOrderItem.setPurchaseCount(purchaseCount);
        }
        qrPackageSapOrderItemMapper.updateById(sapOrderItem);
    }

    private void  saveQrPackageSapOrderItem(QrPackageSapOrderItem sapOrderItem) {

        //采购数量
        long newPurchaseCount = sapOrderItem.getPurchaseCount() == null ? 0 : sapOrderItem.getPurchaseCount();

        //根据sap订单号查询物料订单明细信息
        QrPackageSapOrderItem qrPackageSapOrderItem = getQrPackageSapOrderItemInfo(sapOrderItem.getSapOrderNo(),sapOrderItem.getSapMaterialId());
        long oldPurchaseCount = qrPackageSapOrderItem.getPurchaseCount() == null ? 0 : qrPackageSapOrderItem.getPurchaseCount();

        if (ObjectUtil.isNotNull(qrPackageSapOrderItem)) {
            log.info("Sap物料订单明细订单号:{}sap物料id:{}已存在",sapOrderItem.getSapOrderNo(),sapOrderItem.getSapMaterialId());

            //根据订单号和物料ID查询明细信息 如果存在就累加采购数量
            qrPackageSapOrderItem.setPurchaseCount(oldPurchaseCount + newPurchaseCount);
            qrPackageSapOrderItemMapper.updateById(qrPackageSapOrderItem);

        }

        qrPackageSapOrderItemMapper.insert(sapOrderItem);
    }

    private QrPackageSapOrderItem getQrPackageSapOrderItemInfo(String sapOrderNo,String sapMaterialId) {

        QueryWrapper<QrPackageSapOrderItem> qw = new QueryWrapper<>();
        qw.eq("sap_order_no", sapOrderNo);
        qw.eq("sap_material_id", sapMaterialId);
        qw.eq("is_delete", 0);

        //根据sap订单号查询物料订单明细信息
        QrPackageSapOrderItem qrPackageSapOrderItem = qrPackageSapOrderItemMapper.selectOne(qw);

        return qrPackageSapOrderItem;

    }

}
